// Copyright 2011 Dmitry Chestnykh. All rights reserved.
// Use of this source code is governed by a MIT-style
// license that can be found in the LICENSE file.

package captcha

import "testing"

func BenchmarkNewImage(b *testing.B) {
	d := RandomDigits(DefaultLen)
	id := randomID()

	for b.Loop() {
		NewImage(id, d, StdWidth, StdHeight)
	}
}
